/***
*
*	Copyright (c) 1996-2001, Valve LLC. All rights reserved.
*
*	This product contains software technology licensed from Id
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc.
*	All Rights Reserved.
*
*   This source code contains proprietary and confidential information of
*   Valve LLC and its suppliers.  Access to this code is restricted to
*   persons who have executed a written SDK license with Valve.  Any access,
*   use or distribution of this code by or to any unlicensed person is illegal.
*
****/

#ifdef MOBILE_HACKS
#include	"extdll.h"
#include	"util.h"
#include	"cbase.h"
#include	"monsters.h"
#include	"talkmonster.h"
#include	"schedule.h"
#include	"defaultai.h"
#include	"scripted.h"
#include	"animation.h"
#include	"soundent.h"
#include	"scientist.h"

class CCivScientist : public CScientist
{
public:
	void Spawn(void);
	void Precache(void);

	BOOL CanHeal(void);
};

// LINK_ENTITY_TO_CLASS(monster_civ, CCivScientist);
extern "C" EXPORT void monster_civ( entvars_t *pev )
{
	if( g_iModType == MOD_TOT )
		GetClassPtr( (CCivScientist *)pev );
}

//=========================================================
// Spawn
//=========================================================
void CCivScientist::Spawn(void)
{
	Precache();

	SET_MODEL(ENT(pev), "models/civ_sci.mdl");
	UTIL_SetSize(pev, VEC_HUMAN_HULL_MIN, VEC_HUMAN_HULL_MAX);

	pev->solid = SOLID_SLIDEBOX;
	pev->movetype = MOVETYPE_STEP;
	m_bloodColor = BLOOD_COLOR_RED;
	pev->health = gSkillData.scientistHealth;
	pev->view_ofs = Vector(0, 0, 50);// position of the eyes relative to monster's origin.
	m_flFieldOfView = VIEW_FIELD_WIDE; // NOTE: we need a wide field of view so scientists will notice player and say hello
	m_MonsterState = MONSTERSTATE_NONE;

	//	m_flDistTooFar		= 256.0;

	m_afCapability = bits_CAP_HEAR | bits_CAP_TURN_HEAD | bits_CAP_OPEN_DOORS | bits_CAP_AUTO_DOORS | bits_CAP_USE;

	// White hands
	pev->skin = 0;

	if (pev->body == -1)
	{// -1 chooses a random head
		pev->body = RANDOM_LONG(0, NUM_SCIENTIST_HEADS - 1);// pick a head, any head
	}

	// Luther is black, make his hands black
	if (pev->body == HEAD_LUTHER)
		pev->skin = 1;

	MonsterInit();
	SetUse(&CCivScientist::FollowerUse);
}


//=========================================================
// Precache - precaches all resources this monster needs
//=========================================================
void CCivScientist::Precache(void)
{
	CScientist::Precache();

	PRECACHE_MODEL("models/civ_sci.mdl");
}

BOOL CCivScientist::CanHeal(void)
{
	return FALSE;
}


//=========================================================
// Dead Scientist PROP
//=========================================================
class CDeadCivScientist : public CDeadScientist
{
public:
	void Spawn(void);

	static const char *m_szPoses[7];
};

const char *CDeadCivScientist::m_szPoses[] = { "lying_on_back", "lying_on_stomach", "dead_sitting", "dead_hang", "dead_table1", "dead_table2", "dead_table3" };

LINK_ENTITY_TO_CLASS(monster_civ_dead, CDeadCivScientist);

//
// ********** DeadScientist SPAWN **********
//
void CDeadCivScientist::Spawn()
{
	PRECACHE_MODEL("models/civ_sci.mdl");
	SET_MODEL(ENT(pev), "models/civ_sci.mdl");

	pev->effects = 0;
	pev->sequence = 0;
	// Corpses have less health
	pev->health = 8;//gSkillData.scientistHealth;

	m_bloodColor = BLOOD_COLOR_RED;

	if (pev->body == -1)
	{// -1 chooses a random head
		pev->body = RANDOM_LONG(0, NUM_SCIENTIST_HEADS - 1);// pick a head, any head
	}
	// Luther is black, make his hands black
	if (pev->body == HEAD_LUTHER)
		pev->skin = 1;
	else
		pev->skin = 0;

	pev->sequence = LookupSequence(m_szPoses[m_iPose]);
	if (pev->sequence == -1)
	{
		ALERT(at_console, "Dead scientist with bad pose\n");
	}

	//	pev->skin += 2; // use bloody skin -- UNDONE: Turn this back on when we have a bloody skin again!
	MonsterInitDead();
}
#endif // MOBILE_HACKS
